#include <Wire.h>
#include <AHTxx.h>

#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>
#include <PubSubClient.h>

const char* ssid = "YOUR SSID";
const char* password = "YOUR WIFI PASSWORD";

const char* mqtt_server = "192.168.x.x"; //your home assistant IP
const int mqtt_port = 1883;
const char* mqtt_user = "your mqtt username";
const char* mqtt_password = "your mqtt password";
const char* room_name = "room name of sensor"; // Hard-coded room name

float ahtValue;                               //to store T/RH result

AHTxx aht20(AHTXX_ADDRESS_X38, AHT2x_SENSOR); //sensor address, sensor type

ESP8266WiFiMulti WiFiMulti;
WiFiClient espClient;
PubSubClient client(espClient);

void setup() {
  Serial.begin(74880); // This allows you to see ESP01 start data.
  WiFi.mode(WIFI_STA);
  WiFiMulti.addAP(ssid, password);
  client.setServer(mqtt_server, mqtt_port);

  //GPIO0 is SDA, GPIO2 is SCL
  while (aht20.begin(0, 2, 100000) != true)
  {
    Serial.println(F("AHT2x not connected or fail to load calibration coefficient")); //(F()) save string to flash & keeps dynamic memory free
    delay(5000);
  }

  Serial.println(F("AHT20 OK"));
}

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("ESP8266Client", mqtt_user, mqtt_password)) {
      Serial.println("connected");
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}

void loop() {
  wl_status_t status = WiFiMulti.run(3000);
  if (status == WL_CONNECTED) {
    if (!client.connected()) {
      reconnect();
    }
    client.loop();

    ahtValue = aht20.readTemperature(); //read 6-bytes via I2C, takes 80 milliseconds

    Serial.print(F("Temperature: "));
    
    if (ahtValue != AHTXX_ERROR) //AHTXX_ERROR = 255, library returns 255 if error occurs
    {
      Serial.print(ahtValue);
      Serial.println(F(" +-0.3C"));
      
      // Publish to MQTT
      String topic = String("homeassistant/sensor/") + room_name + String("/temperature");
      String payload = String(ahtValue);
      client.publish(topic.c_str(), payload.c_str());
    }
    else
    {
      printStatus(); //print temperature command status
    }
    //delay(10000);
  }
  else {
    Serial.println(status);
  }

  // USE THIS FOR USB POWERED
  delay(10000);

  // USE THIS FOR BATTERY POWERED
  //delay(500);
  //ESP.deepSleep(300e6);
}

/**************************************************************************/
/*
    printStatus()

    Print last command status
*/
/**************************************************************************/
void printStatus()
{
  switch (aht20.getStatus())
  {
    case AHTXX_NO_ERROR:
      Serial.println(F("no error"));
      break;

    case AHTXX_BUSY_ERROR:
      Serial.println(F("sensor busy, increase polling time"));
      break;

    case AHTXX_ACK_ERROR:
      Serial.println(F("sensor didn't return ACK, not connected, broken, long wires (reduce speed), bus locked by slave (increase stretch limit)"));
      break;

    case AHTXX_DATA_ERROR:
      Serial.println(F("received data smaller than expected, not connected, broken, long wires (reduce speed), bus locked by slave (increase stretch limit)"));
      break;

    case AHTXX_CRC8_ERROR:
      Serial.println(F("computed CRC8 not match received CRC8, this feature supported only by AHT2x sensors"));
      break;

    default:
      Serial.println(F("unknown status"));    
      break;
  }
}
